import { cloneDeep, pick, set } from "lodash";
import { Exception, queenApi } from "queenjs";
import { EditorModule } from "..";
import { DesignComp } from "../../objects/DesignTemp/DesignComp";
import { Matrix } from "../../controllers/TransferCtrl/Matrix";
import { ICompKeys, Layout } from "../../typings";

export const editActions = EditorModule.action({
  // 添加组件到画布
  async addCompToDesign(compKey: ICompKeys, index?: number) {
    if (!this.store.currStreamCardId) {
      //必须选中一个streamCard
      return;
    }

    if (compKey == "Container") {
      index = this.store.streamCardIds.indexOf(this.store.currStreamCardId) + 1;
      const compId = await this.store.insertDesignContent(compKey, index);
      this.actions.pickComp(compId);
      return;
    }

    // if (
    //   index === undefined &&
    //   this.store.currComp?.compKey === "Container"
    //   // this.store.pageCompIds.includes(this.store.currComp.id)
    // ) {

    const compId = await this.store.insertCompContainer(
      compKey,
      this.store.currStreamCard
    );
    this.actions.pickComp(compId);
    this.actions.setCompPosition(this.store.currComp);
    // } else {
    //   compId = await this.store.insertDesignContent(compKey, index);
    //   this.actions.pickComp(compId);
    // }
  },
  // 切换当前组件
  pickComp(compId: string) {
    const { store, helper } = this;
    // 组合模式下，切换组件
    if (store.currCompId && store.groupModeStatus) {
      const enableGroupIds = helper
        .findParentComp(compId)
        ?.getChildIds() as string[];
      const comps = helper.getCompTrees(compId);
      while (comps.length) {
        const comp = comps.pop() as DesignComp;
        const index = store.groupIds.indexOf(comp.id);
        if (index >= 0) {
          const groupIds = [...store.groupIds];
          groupIds.splice(index, 1);
          store.setGroupIds(groupIds);
        } else if (enableGroupIds.includes(comp.id)) {
          store.groupIds.push(comp.id);
          return;
        }
      }
      return;
    }
    // let nextCompId = compId;
    // if (this.store.isEditPage) {
    //   const comps = this.helper.getCompTrees(compId);
    //   nextCompId = comps[1].id;
    // }
    if (this.store.currCompId == compId) {
      return;
    }
 
    this.store.setCurrComp(compId);
    if (this.store.currCompId == this.store.currStreamCardId) {
      this.controls.transferCtrl.destroy();
    }
  },
  // 切换到父组件
  pickParentComp(compId: string) {
    const parentComp = this.helper.findParentComp(compId);
    parentComp && this.store.setCurrComp(parentComp.id);
  },
  // 删除组件
  removeComp(compId: string) {
    if (this.helper.isCompCanDelete(compId)) {
      if (compId === this.store.currCompId) {
        this.store.setCurrComp("root");
      }
      this.store.deleteComp(compId);
    }
  },

  async removeStreamCard(compId: string) {
    await queenApi.showConfirm({ title: "删除", content: "确认删除当前内容?" });

    if (this.store.streamCardIds.length < 2) {
      return;
    }
    let nextCard = this.store.currStreamCardId;
    if (compId == this.store.currStreamCardId) {
      const i = this.store.streamCardIds.indexOf(compId);
      nextCard = this.store.streamCardIds[i + 1];
      if (!nextCard) {
        nextCard = this.store.streamCardIds[i - 1];
      }
    }
    this.store.deleteComp(compId);
    this.store.setCurrComp(nextCard);
  },
  // 移动组件顺序
  moveComp(selIndex: number, targetIndex: number) {
    if (selIndex === targetIndex) return;
    this.store.moveComp(selIndex, targetIndex);
  },

  // 保存容器为组件
  async saveAsComp(comp: DesignComp) {
    try {
      // 组件封面
      const blob = await this.helper.screenshot({
        element: comp.$el,
      });
      const thumbnail = URL.createObjectURL(blob);
      const title = await queenApi.showInput({
        title: "保存到我的组件",
        defaultValue: this.controls.compUICtrl.state.components.get(
          comp.compKey
        )?.name,
      });

      const data = {
        title,
        thumbnail,
        compMap: cloneDeep(this.store.designData.compMap),
      };
      this.helper.clearUnusedComps(data.compMap, comp.id);
      data.compMap.root = data.compMap[comp.id];
      data.compMap.root.id = "root";
      delete data.compMap[comp.id];

      queenApi.showLoading("保存中");
      await this.controls.uploader.uploadBlobs(data);
      await this.https.createComp(data);
      queenApi.messageSuccess("保存成功");
    } catch (error: any) {
      throw Exception.error(error.toString());
    } finally {
      queenApi.hideLoading();
    }
  },

  // 保存项目
  async saveDesign() {
    try {
      // 清除无用组件
      this.helper.clearUnusedComps(this.store.designData.compMap);
      // 封面截屏
      if (!this.store.designData.thumbnail) {
        await this.actions.updateThumbnailByScreenshot();
      }
      queenApi.showLoading("保存中");
      await this.controls.uploader.uploadBlobs(this.store.designData);
      await this.https[this.store.isEditPage ? "saveDesign" : "saveComp"](
        this.store.designData
      );
      queenApi.messageSuccess("保存成功");
    } catch (error: any) {
      throw Exception.error(error.toString());
    } finally {
      queenApi.hideLoading();
    }
  },

  // 截屏保存封面
  async updateThumbnailByScreenshot(autoSave?: boolean) {
    try {
      const rootComp = this.helper.findRootComp();
      if (!rootComp) return;
      queenApi.showLoading("截屏中");
      const blob = await this.helper.screenshot({
        element: rootComp.$el,
        ratio: this.store.isEditComp ? 0 : 1,
      });
      const thumbnail = URL.createObjectURL(blob);
      this.store.setDesignThumbnail(thumbnail);

      if (autoSave) {
        await this.controls.uploader.uploadBlobs(this.store.designData);
        await this.https[this.store.isEditPage ? "saveDesign" : "saveComp"](
          pick(this.store.designData, ["_id", "thumbnail"])
        );
        queenApi.messageSuccess("保存成功");
      }
    } catch (error: any) {
      throw Exception.error(error.toString());
    } finally {
      queenApi.hideLoading();
    }
  },

  // 设置组件变换
  setCompTransform(comp: DesignComp, transform: Layout["transform"]) {
    if (!comp) return;

    comp.layout.transform = transform;
    console.log(comp);
  },

  updateCompData(comp: DesignComp, path: string, value: any) {
    set(comp, path, value);
  },

  // 设置组件浮动
  setCompPosition(comp: DesignComp) {
    comp.layout.position =
      comp.layout.position === "absolute" ? undefined : "absolute";
  },
  // 设置组件显示隐藏
  setCompVisible(comp: DesignComp) {
    comp.layout.visible = comp.layout.visible === false ? true : false;
  },
  // 清除组件变换
  clearCompTransform(comp: DesignComp) {
    comp.layout.margin = "";
    comp.layout.transform = undefined;
  },
  // 设置组件锁定状态
  setCompLock(comp: DesignComp) {
    console.log(comp);
  },
  // 设置组件层级
  setCompLayer(comp: DesignComp, offset: number) {
    comp.layout.zIndex = Math.min(
      Math.max((comp.layout.zIndex || 0) + offset, 0),
      99
    );
  },
  // 宽度铺满
  fullCompWidth(comp: DesignComp) {
    comp.layout.size || (comp.layout.size = []);
    comp.layout.size[0] = 750;
  },
  //
  setCompAlign(comp: DesignComp, align: string) {
    comp.layout.alignSelf = align;
    if (comp.layout.transform?.x) {
      comp.layout.transform.x = 0;
    }
  },

  // 开启组合模式
  enableGroupMode() {
    this.store.setGroupIds(
      this.store.currCompId ? [this.store.currCompId] : []
    );
    this.store.setGroupMode(true);
  },
  // 关闭组合模式
  async disableGroupMode() {
    const groupId = await this.actions.combineGroupComps();
    if (groupId) {
      this.store.setCurrComp(groupId);
    }
    this.store.setGroupIds([]);
    this.store.setGroupMode(false);
  },
  // 合并组件
  async combineGroupComps() {
    const { helper, store } = this;
    const { groupIds } = store;
    if (groupIds.length < 2) return;
    const compsRect: Record<
      string,
      { t: number; l: number; r: number; b: number }
    > = {};
    const parentComp = helper.findParentComp(groupIds[0]) as DesignComp;
    const parentRect = parentComp.$el.getBoundingClientRect();
    groupIds.forEach((id) => {
      const comp = helper.findComp(id) as DesignComp;
      const itemRect = comp.$el.getBoundingClientRect();
      compsRect[id] = {
        t: itemRect.top - parentRect.top,
        l: itemRect.left - parentRect.left,
        r: itemRect.right - parentRect.left,
        b: itemRect.bottom - parentRect.top,
      };
    });

    const confVals = Object.values(compsRect);
    const groupConf = {
      t: Math.min(...confVals.map((d) => d.t)),
      l: Math.min(...confVals.map((d) => d.l)),
      r: Math.max(...confVals.map((d) => d.r)),
      b: Math.max(...confVals.map((d) => d.b)),
    };

    const groupId = await this.store.insertCompContainer("Group", parentComp);

    const groupComp = helper.findComp(groupId) as DesignComp;

    groupComp.layout = {
      size: [
        helper.pxToDesignSize(groupConf.r - groupConf.l),
        helper.pxToDesignSize(groupConf.b - groupConf.t),
      ],
      position: "absolute",
      transform: {
        x: helper.pxToDesignSize(groupConf.l),
        y: helper.pxToDesignSize(groupConf.t),
      },
    };

    groupIds.forEach((id) => {
      const comp = helper.findComp(id) as DesignComp;
      comp.layout.transform || (comp.layout.transform = {});
      comp.layout.transform.x =
        (comp.layout.transform.x || 0) - (groupComp.layout.transform?.x || 0);

      comp.layout.transform.y =
        (comp.layout.transform.y || 0) - (groupComp.layout.transform?.y || 0);
    });

    groupComp.children.default = parentComp.children.default?.filter((d) =>
      groupIds.includes(d)
    );
    parentComp.children.default = parentComp.children.default?.filter(
      (d) => !groupIds.includes(d)
    );

    return groupId;
  },
  // 取消打组
  cancelGroupComps(groupComp: DesignComp) {
    const { helper } = this;
    const groupChildIds = groupComp.children.default as string[];

    const parentComp = helper.findParentComp(groupComp.id) as DesignComp;

    const parentMatrix = new Matrix();

    groupChildIds.forEach((id) => {
      const comp = helper.findComp(id) as DesignComp;

      parentMatrix.setFormDiv(groupComp.$el);
      const originArr = window
        .getComputedStyle(groupComp.$el)
        .transformOrigin.split(" ")
        .map(parseFloat);

      const porigin = new Matrix();
      porigin.translate(originArr[0], originArr[1]);
      const invOrigin = new Matrix();
      invOrigin.translate(-originArr[0], -originArr[1]);

      const childOrigArr = window
        .getComputedStyle(comp.$el)
        .transformOrigin.split(" ")
        .map(parseFloat);
      const corigin = new Matrix();
      corigin.translate(childOrigArr[0], childOrigArr[1]);
      const cinvOrigin = new Matrix();
      cinvOrigin.translate(-childOrigArr[0], -childOrigArr[1]);

      const childMatrix = new Matrix();
      childMatrix.setFormDiv(comp.$el);

      const result = cinvOrigin
        .multipy(porigin)
        .multipy(parentMatrix)
        .multipy(invOrigin)
        .multipy(corigin)
        .multipy(childMatrix);

      comp.layout.transform || (comp.layout.transform = {});
      comp.layout.transform.x = helper.pxToDesignSize(result.getX());
      comp.layout.transform.y = helper.pxToDesignSize(result.getY());
      comp.layout.transform.s = result.getScale();
      comp.layout.transform.r = result.getRotate();
    });

    const childIds = [...(parentComp.children.default as string[])];

    const groupIndex = childIds.findIndex((id) => groupComp.id === id);
    childIds.splice(groupIndex, 1, ...groupChildIds);

    parentComp.children.default = childIds;
    this.store.setCurrComp(groupChildIds[0]);
  },
});
